package il.ac.tau.cs.dqsq.common;

import il.ac.tau.cs.dqsq.qsq2axml.Qsq2AxmlTranslator;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import org.jdom.Element;

/**
 * @author pettelno
 *
 * A dDatalog relation.
 * 
 */
public class Relation
{
    private String name = null;
    private String adornment = null;
    private int numOfAttributes;
    private List tuples = null;

    public Relation()
    {   	
    }
    
    public Relation(String name, String adornment, int numOfAttributes, List tuples)
    {
    	this.name = name;
    	this.adornment = adornment;
    	this.numOfAttributes = numOfAttributes;
    	this.tuples = tuples;
    }
    
    public Relation(RelationOccurrence relOccurrence)
    {
    	this.name = relOccurrence.getName();
    	this.adornment = relOccurrence.getAdornment();
    	this.numOfAttributes = relOccurrence.numberOfAttributes();
    }
    
    public void read(Element relationElem)
    {
        name = relationElem.getAttributeValue("name");
        Element tuplesElem = relationElem.getChild("tuples");
        if(tuplesElem != null)
        {
            this.tuples = tuplesElem.cloneContent();
            this.numOfAttributes = tuplesElem.getChild("t").getChildren().size();
        }
        else
        	this.numOfAttributes = 0;
    }

    public String getShortName()
    {
    		return this.name;
    }
    
    public String getName()
    {
    	// Returns the full name of the relation (with the adornmant, if exists).
    	// E.g., r_bf.
    	if(this.adornment == null)
//    		return this.name;
    		return this.name + "_" + Qsq2AxmlTranslator.getQueryIndex();
    	
    	return this.name + "_" + this.adornment + "_" + Qsq2AxmlTranslator.getQueryIndex();
//    	return this.name + "_" + this.adornment;
    }
    
    public List getTuples()
    {
    	return this.tuples;
	}
    
    public int getnumOfAttributes()
    {
    	return this.numOfAttributes;
	}

    public void concatName(String str)
    {
    	this.name += str;
    }
    
    public ArrayList serialize()
    {
    	// Returns an ArrayList of ArrayLists containing the tuples that
    	// exist in the relation.
    	
    	ArrayList serializedRelation = new ArrayList();
    	
    	Iterator tuplesIter = this.tuples.iterator();
    	while(tuplesIter.hasNext())
    	{
//    		Element tupleElem = (Element)tuplesIter.next();
    		Element tupleElem = null;
    		Object tuple = tuplesIter.next();
    		if(tuple instanceof Element)
    		{
    			tupleElem = (Element)tuple;
	    		List tupleValuesElems = tupleElem.getChildren();
	    		ArrayList outTupleValues = new ArrayList();
	    		Iterator tupleValuesIter = tupleValuesElems.iterator();
	    		while(tupleValuesIter.hasNext())
	    		{
	    			String val = ((Element)tupleValuesIter.next()).getValue();
	    			outTupleValues.add(val);
	    		}
	    		serializedRelation.add(outTupleValues);
    		}
    	}
    	
    	return serializedRelation;
    }
}
