package il.ac.tau.cs.dqsq.datalog2qsq;

import java.net.URL;
import localhost.axml.services.Datalog2QSQ.PeerServiceLocator;
import localhost.axml.services.PeerAccessPoint.PeerAccessPointServiceLocator;
import localhost.axml.services.ActivityService.ActivityServiceServiceLocator;
import il.ac.tau.cs.dqsq.qsq2axml.Qsq2AxmlTranslator;
import il.ac.tau.cs.dqsq.util.Logger;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.HashMap;
import localhost.axml.services.Datalog2QSQ.Datalog2QSQSoapBindingStub;

/**
 * @author pettelno
 * 
 * The entry point of the calls from the JSPs that forms the Web UI of
 * the dQSQ system. A single JspQuery is used for all the queries, which
 * are issued sequentially (simultaneous queries are not supported).
 *
 */
public class JspQuery
{
	private String query = null;
	private String queryName = null;
	private String peerURL = null;
	private String peerName = null;
	private List queryResult = new ArrayList();
	private List queryVariables = new ArrayList();
	private HashMap datalogProgram = null;
	private List queryHistory = new ArrayList();
	
	public void init(StringBuffer requestURL)
	{
		setPeerURL(requestURL.toString());
	}
	
	public void runQuery(String query) throws Exception
	{		
		setMembers(query);
		if(!queryHistory.contains(query))
			queryHistory.add(query);
		
    	PeerServiceLocator locator = new PeerServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
			localhost.axml.services.Datalog2QSQ.Peer remotePeer =
				locator.getDatalog2QSQ(serviceURL);

			// Set the timeout of the web service to infinite.
			// Default timeout is 1 minute.
			((Datalog2QSQSoapBindingStub)remotePeer).setTimeout(0);

			remotePeer.runQuery(query);
		}
		catch(Exception e)
		{
			clear();
			sendLogMessage("Failed to run query " + query + ".", Logger.LOG_LEVEL_ERROR);
			throw e;
		}    	    			
	}

	private void setQueryVariables()
	{
		int varsBeginIndex = this.query.indexOf('(');
		int varsEndIndex = this.query.indexOf(')');
		String vars = this.query.substring(varsBeginIndex + 1, varsEndIndex);
		StringTokenizer varsTokenizer = new StringTokenizer(vars, ",");
		while (varsTokenizer.hasMoreTokens())
			this.queryVariables.add(varsTokenizer.nextToken());
	}
	
	private String getPeerName(String query)
	{
		// Example: For the query q@P(y) :- ancestor@P("Joyce", y), the peer
		// name is P.
		
		String[] splittedQuery = query.split("@");
		return splittedQuery[1].substring(0, 1);
	}
	
	public String getQuery()
	{
		return this.query;
	}
	
	public void initQueryResult()
	{
		PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
		try
		{
			// Run the query using the query web service at PeerAccessPoint.
			URL serviceURL = new URL(this.peerURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);
			String queryFullName = this.queryName + "_" + Qsq2AxmlTranslator.getQueryIndex();
//			String queryFullName = this.queryName;
			Element replyElem = service.query("select tuple from tuple in " + queryFullName + "//data/tuples/t;");

			// Convert the reply from Dom Element to a List of Lists.
			// For example, the reply 
			// <result>
			//  <t>
			//   <at1>Noam</at1>
			//   <at2>Pettel</at2>
			//  </t>
			//  <t>
			//   <at1>Tal</at1>
			//   <at2>Zur</at2>
			//  </t>
			//  <t>
			//   <at1>Yotam</at1>
			//   <at2>Shtossel</at2>
			//  </t>
			//
			// forms a List of 3 Lists: { {Noam,Pettel}, {Tal,Zur}, {Yotam,Shtossel} }.
			this.queryResult.clear();
			NodeList tupleElems = replyElem.getElementsByTagName("t");
			int tuplesNum = tupleElems.getLength();
			for(int tupleIndex = 0; tupleIndex < tuplesNum; tupleIndex++)
			{
				Element tupleElem = (Element)tupleElems.item(tupleIndex);
				NodeList attributeElems = tupleElem.getElementsByTagName("*");
				int attributesNum = attributeElems.getLength();
				List attributeList = new ArrayList(attributesNum);
				for(int attributeIndex = 0; attributeIndex < attributesNum; attributeIndex++)
				{
					Element attributeElem = (Element)attributeElems.item(attributeIndex);					
					String attributeValue = attributeElem.getFirstChild().getNodeValue();
					attributeList.add(attributeIndex, attributeValue);
				}
				
				this.queryResult.add(tupleIndex, attributeList);
			}
		}
		catch(Exception e)
		{
			sendLogMessage("Failed to get query result for query " + this.query + ".",
						   Logger.LOG_LEVEL_ERROR);
		}
	}
	
	public String getQueryResult(int elementIndex, int attributeIndex)
	{
		// In the above example, a call to this method with (2,1) will
		// return "Tal".
		List elementList = (List)(this.queryResult.get(elementIndex));
		return (String)(elementList.get(attributeIndex));
	}
	
	public int getNumOfQueryResultElements()
	{
		// In the above example, will return 3.
		return this.queryResult.size();
	}
	
	public int getNumOfAttributesInElement()
	{
		// In the above example, will return 2.
		return this.queryVariables.size();
		
		/*
		// In the above example, will return 2.
		if(this.queryResult.isEmpty())
			return 0;
		List elementList = (List)(this.queryResult.get(0));
		return elementList.size();
		*/
	}
	
	public String getVariableNameAtIndex(int index)
	{
		// For the query q@P(y) :- r@P("1", y), a call to this method with
		// index=0 will return "y".
		return (String)this.queryVariables.get(index);
	}
	
	public boolean isQueryComputationFinished()
	{
		// Find out if query computation is finished by calling the
		// isTerminated web service at ActivityService.
    	ActivityServiceServiceLocator locator = new ActivityServiceServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/ActivityService");
			localhost.axml.services.ActivityService.ActivityService activityService =
				locator.getActivityService(serviceURL);
			return activityService.isTerminated(this.peerName);			
		}
		catch(Exception e)
		{
			sendLogMessage("Failed to check if query computation is finished.",
						   Logger.LOG_LEVEL_ERROR);
			return false;
		}    	    			
	}

    public void terminateQuery()
    {
    	clear();
    	
    	PeerServiceLocator locator = new PeerServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
			localhost.axml.services.Datalog2QSQ.Peer remotePeer =
				locator.getDatalog2QSQ(serviceURL);
			remotePeer.terminateQuery();			
		}
		catch(Exception e)
		{
			sendLogMessage("Failed to terminate query " + query + ".",
						   Logger.LOG_LEVEL_ERROR);
		}    	    			    	
    }
    
    private void clear()
    {
    	this.query = null;
		this.queryResult.clear();
    	this.queryVariables.clear();
    }
    
    private void setMembers(String query)
    {
		this.query = query;
		setQueryVariables();

		// Set peer name and query name.
		// Example: For the query q@P(y) :- ancestor@P("Joyce", y),
		//the peer name is P and the query name is q.		
		String[] splittedQuery = query.split("@");
		this.queryName = splittedQuery[0];
		this.peerName = splittedQuery[1].substring(0, 1);		
    }
    
    private void setPeerURL(String requestURL)
    {
    	// The request URL is <peerURL>/jsp/dQSQ/runQuery.jsp, for
    	// example http://localhost:8080/axml/jsp/dQSQ/runQuery.jsp.
    	// Extrace the peer URL.
    	String[] splittedRequestURL = requestURL.split("/jsp");
    	this.peerURL = splittedRequestURL[0];
    }

    public HashMap getDatalogProgram()
    {
    	// The datalog program is retrieved only once and keeped as a
    	// member.
    	if(this.datalogProgram == null)
    	{
	    	PeerServiceLocator locator = new PeerServiceLocator();
			try
			{
				URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
				localhost.axml.services.Datalog2QSQ.Peer remotePeer =
					locator.getDatalog2QSQ(serviceURL);
				this.datalogProgram = remotePeer.getDatalogProgram();
			}
			catch(Exception e)
			{
				sendLogMessage("Failed to get Datalog program.", Logger.LOG_LEVEL_ERROR);
			}
    	}
    	
    	return this.datalogProgram;
    }


    public Object[] getQsqRewritingOutput()
    {
    	PeerServiceLocator locator = new PeerServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
			localhost.axml.services.Datalog2QSQ.Peer remotePeer =
				locator.getDatalog2QSQ(serviceURL);
			return remotePeer.getQsqRewritingOutput();
		}
		catch(Exception e)
		{
			sendLogMessage("Failed to get Query-Sub-Query rewriting output.", Logger.LOG_LEVEL_ERROR);
			return null;
		}
    }
  
    public List getQueryHistory()
    {
    	return this.queryHistory;
    }
    
    private void sendLogMessage(String message, int severity)
    {
    	// Send the log message to the Peer with a Web service.
    	// Calling Logger.sendMessage directly will fail to get the
    	// output dir, since we don't have the AXML context.
    	PeerServiceLocator locator = new PeerServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
			localhost.axml.services.Datalog2QSQ.Peer remotePeer =
				locator.getDatalog2QSQ(serviceURL);
			remotePeer.sendLogMessage(message, severity);
		}
		catch(Exception e)
		{
		}    	
    }

    public String getPeerName()
    {
    	if(this.peerName == null)
    	{
        	PeerServiceLocator locator = new PeerServiceLocator();
			try
			{
				URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
				localhost.axml.services.Datalog2QSQ.Peer peer =
					locator.getDatalog2QSQ(serviceURL);
				this.peerName = peer.getPeerName();
			}
			catch(Exception e)
			{
				sendLogMessage("Failed to get peer name.", Logger.LOG_LEVEL_ERROR);
			}    	
    	}
    
    	return this.peerName;
    }

    public long getQueryComputationTime() throws Exception
    {    
    	PeerServiceLocator locator = new PeerServiceLocator();
		try
		{
			URL serviceURL = new URL(this.peerURL + "/services/Datalog2QSQ");
			localhost.axml.services.Datalog2QSQ.Peer peer =
				locator.getDatalog2QSQ(serviceURL);
			return peer.getQueryComputationTime();
		}
		catch(Exception e)
		{
			sendLogMessage("Failed to get query computation time.", Logger.LOG_LEVEL_ERROR);
			throw e;
		}    	    	
    }  
}
