package il.ac.tau.cs.dqsq.datalog2qsq;

import il.ac.tau.cs.dqsq.common.RelationOccurrence;
import il.ac.tau.cs.dqsq.common.Rule;
import java.util.ArrayList;
//import java.util.Collection;
//import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.HashMap;

/**
 * @author pettelno
 *
 * Extends the basic Rule class to represent a rule that is being
 * rewritten as a part of the dQSQ computation process.
 *  
 */
class RewritableRule extends Rule
{
	RewritableRule(RewritableRule rule)
	{
		super(rule);
	}
	
	RewritableRule()
	{
	}
	
	RewritableRule(String ruleAsString)
	{
		super(ruleAsString);
	}

	RewritableRule(HashMap ruleAsHashMap)
	{
		super(ruleAsHashMap);
	}
	
	void setAdornment(String adornment, Set intensionalRelations)
	{
		List boundedVariables = new ArrayList();
		
		setHeadAdornment(adornment, boundedVariables);
		setBodyAdornment(intensionalRelations, boundedVariables);		
	}

/*	
	private void setRuleAdornments(String adornment, Set intensionalRelations)
	{
		List boundedVariables = new ArrayList();
		
		setHeadAdornment(adornment, boundedVariables);
		setBodyAdornment(intensionalRelations, boundedVariables);		
	}
*/
	
	private void setHeadAdornment(String adornment, List boundedVariables)
	{
		// The adornment of the relation in the head is the adornment
		// given as a parameter, i.e. the adornment with which the 
		// rewriting of the rule started.
		this.head.setAdornment(adornment);
		int numOfHeadsVars = adornment.length();
		for(int index = 0; index < numOfHeadsVars; index++)
		{
			if(adornment.charAt(index) == 'b')
				boundedVariables.add(this.getHeadsAttributeName(index));
		}		
	}

	private void setBodyAdornment(Set intensionalRelations, List boundedVariables)
	{
		int numOfRelationsInBody = this.body.size();
		for(int index = 0; index < numOfRelationsInBody; index++)
		{
			RelationOccurrence rel = getRelationFromBody(index);
			
			// Set adornment only for local relations which are intensional,
			// and for remote relations. For a remote relation, we don't know
			// if it is intensional or extensional, so we treat it as
			// intensional. The peer that hosts this relation will fix
			// it if necessary.
			
			// The if is commented out. At this point we need the
			// adornment also for local extensional relations,
			// because it is used to sort the relations in the
			// rule body. The extra adornments are removed 
			// afterwards.
//			if((intensionalRelations.contains(rel.getName())) ||
//			   (!getHostingPeer().equals(rel.getPeerName())))
					setRelationAdornment(rel, boundedVariables);
		}		
	}
	
	private void setRelationAdornment(
			RelationOccurrence rel, List boundedVariables)
	{
		String relationAdornment = "";
		
		int numOfVarsInRelation = rel.numberOfAttributes();
		String curVarName = null;
		for(int index = 0; index < numOfVarsInRelation; index++)
		{
			// If the variable appears left to the current position in
			// the rule it is bounded, else, it's free.
			curVarName = rel.getAttributeName(index);
			if(boundedVariables.contains(curVarName))
				relationAdornment += 'b';
			else
			{
				relationAdornment += 'f';
				boundedVariables.add(curVarName);
			}
		}
		
		rel.setAdornment(relationAdornment);
	}
	
    void sortRelationsByBindingLevel()
    {
		// Use "Bubble Sort" to sort the relations in the rule
    	// body by their binding level in reverse order. After
    	// sorting the relation with the highest binding level
    	// will be first. This way the variables get bound as
    	// early as possible to reduce the amount of data to
    	// materialize.
    	// See paper "Web Services Meet Datalog" by Abiteboul &
    	// Milo, section 6 and the book "Foundations of databases",
    	// by Abiteboul et al., RSG example at page 319.
    	
    	int numOfRelationsInBody = getNumOfBodyRelations();
    	if(numOfRelationsInBody < 2)
    		return;
    	
    	for(int extIndex = 0; extIndex < numOfRelationsInBody - 1; extIndex++)
    	{
        	for(int intIndex = 0; intIndex < numOfRelationsInBody - 1; intIndex++)
        	{
        		RelationOccurrence rel1 = getRelationFromBody(intIndex);
        		RelationOccurrence rel2 = getRelationFromBody(intIndex + 1);
        		if(rel1.getBindingLevel() < rel2.getBindingLevel())
        			switchRelationsInBody(intIndex, intIndex + 1);
        	}
    	}
    }
    
    void removeExtraAdornments(Map extensionalRelations)
    {
    	// Remove adornmants of local extensional relations.
    	String relationName = getHeadRelationName();
    	if(extensionalRelations.get(relationName) != null)
    		this.head.clearAdornment();
    	
    	int numOfBodyRelations = getNumOfBodyRelations();
    	for(int index = 0; index < numOfBodyRelations; index++)
    	{
        	relationName = getBodysRelationName(index);
        	if(extensionalRelations.get(relationName) != null)
        		getRelationFromBody(index).clearAdornment();    		
    	}
    }    
}
