package il.ac.tau.cs.dqsq.datalog2qsq;

import java.util.Collection;
import java.util.HashSet;
import org.jdom.Element;

/**
 * @author pettelno
 *
 * Manages a dDatalog rule. Each rule in the original dDatalog program
 * has a single instance of this class. A RewritableRuleManager object
 * keeps the history of adornments of the rule.  
 *
 */
class RewritableRuleManager
{
	private RewritableRule originalRule = null;

	// Keeps all previous adornments with which this rule was rewritten.
	private Collection adornmentsHistory = new HashSet();

	RewritableRuleManager(Element ruleElem)
	{
        this.originalRule = new RewritableRule();
        this.originalRule.read(ruleElem);		
	}
	
	boolean isRewritten(String adornment)
	{
		// Returns true if this rule was previously rewritten with the
		// given adornment.
		return this.adornmentsHistory.contains(adornment);
	}

    void clearAdornmentsHistory()
    {
    	this.adornmentsHistory.clear();
    }

	void addAdornment(String adornment)
	{
		// Remember that the rule was rewritten with this adornment. 
		this.adornmentsHistory.add(adornment);
	}
	
	RewritableRule getRule()
	{
		// Return a deep clone of the original rule.
		return new RewritableRule(this.originalRule);
	}
	
	String getHeadRelationName()
	{
		return this.originalRule.getHeadRelationName();
	}
	
	String getName()
	{
		return this.originalRule.getName();
	}

    public String toString()
    {
    	return this.originalRule.toString();
    }
}
