package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import localhost.axml.services.ActivityService.*;
import il.ac.tau.cs.dqsq.util.Logger;

/**
 * @author pettelno
 *
 * The builder of the activity file, "activity.xml", which is needed for
 * the termination detection algorithm, in a dynamic mode. In this mode the
 * activity file is generated gradually as the dQSQ computation evolves and
 * while the AXML peer is running. 
 * 
 */
class DynamicActivityFile implements IActivityFile
{
	private String localURL = null;
	private String peerName = null;
	
	DynamicActivityFile(String localURL, String peerName, String outputDirName)
		throws IOException
	{
		this.localURL = localURL;
		this.peerName = peerName;
		
        Element root = new Element("activity").setAttribute("terminated", "n");		
        Document outDoc = new Document(root);

        // Serialize it into a file.
        String outputFileName = outputDirName + File.separator + "activity.xml";
        FileOutputStream out = new FileOutputStream(outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(outDoc, out);
        out.flush();
        out.close();		
	}
	
	public void addRelation(String relationName)
	{
    	ActivityServiceServiceLocator locator = new ActivityServiceServiceLocator();
		try
		{
			URL serviceURL = new URL(this.localURL + "/services/ActivityService");
			ActivityService service = locator.getActivityService(serviceURL);
			service.addRelation(this.peerName, relationName);
		}
		catch(Exception e)
		{
			Logger.sendMessage("Failed to add relation to activity file",
							   Logger.LOG_LEVEL_ERROR);
		}    	    			
	}
}
