package il.ac.tau.cs.dqsq.qsq2axml;

import java.util.Iterator;
import java.util.List;
import localhost.axml.services.IdlenessManager.IdlenessManagerServiceLocator;
import java.net.URL;
import il.ac.tau.cs.dqsq.common.RelationOccurrence;
import il.ac.tau.cs.dqsq.util.Logger;
import localhost.axml.services.IdlenessManager.IdlenessManager;

/**
 * @author pettelno
 *
 * Builds the map of relations and their neighbors dynamically, as the
 * dQSQ computation evolves.
 *  
 */
class DynamicNeighborsFile implements INeighborsFile
{
	private String localURL = null;
	private IdlenessManagerServiceLocator locator = null;
	private URL serviceURL = null;
	private IdlenessManager service = null;
	
	DynamicNeighborsFile(String localURL, String peerName) throws Exception
	{
		this.localURL = localURL;
		this.locator = new IdlenessManagerServiceLocator();
		this.serviceURL = new URL(this.localURL + "/services/IdlenessManager");
		this.service = this.locator.getIdlenessManager(serviceURL);
		
		service.init(peerName);		
	}
	
	public void addRelation(String relationName, List neighbors)
	{
		AddRelationToIdlenessManager(relationName);
		AddNeighborsToIdlenessManager(relationName, neighbors);
	}
	
	private void AddRelationToIdlenessManager(String relationName)
	{
		try
		{
			service.addRelation(relationName);
		}
		catch(Exception e)
		{
			Logger.sendMessage("Failed to add relation " + relationName + " to idleness manager. (System error: " + e.getMessage() + ".",
							   Logger.LOG_LEVEL_ERROR);
		}    	    			    	    		
	}
	
	private void AddNeighborsToIdlenessManager(String relationName, List neighbors)
	{
		String neighborName = null;
		
		try
		{
			if(neighbors != null)
			{
				Iterator neighborsIter = neighbors.iterator();
				while(neighborsIter.hasNext())
				{
					RelationOccurrence curNeighborRel = (RelationOccurrence)neighborsIter.next();
					neighborName = curNeighborRel.getAdornedName();
					
					service.addNeighbor(
							relationName,
							neighborName,
							curNeighborRel.getPeerName(),
							curNeighborRel.getServiceShortURL());
				}
	        }
		}
		catch(Exception e)
		{
			Logger.sendMessage("Failed to add neighbor " + neighborName + " to idleness manager. (System error: " + e.getMessage() + ".",
							   Logger.LOG_LEVEL_ERROR);
		}    	    			    	    		
	}
}
