package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.net.URL;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import localhost.axml.services.PeerAccessPoint.PeerAccessPointServiceLocator;
import il.ac.tau.cs.dqsq.util.Logger;
import il.ac.tau.cs.dqsq.util.dQSQUtils;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;

/**
 * @author pettelno
 *
 * A dynamic repository of AXML documents or services. "Dynamic" means that
 * the repositpry is built gradually, when new documents and services can
 * be added while the AXML peer is already running. The repository uses
 * the web services in PeerAccessPoint to add and update documents and
 * services.
 * 
 */
class DynamicRepository implements IRepository
{
    private String outputDirName = null;	
    private String localShortURL = null;
    private Set documents = new HashSet();
    private Set services = new HashSet();
	
	DynamicRepository(String outputDirName)
	{
    	this.outputDirName = outputDirName;
    	this.localShortURL = dQSQUtils.getPeerURL();
	}
	
	public void addDocument(String docName)
	{
        Logger.sendMessage("DynamicRepository.addDocument started", Logger.LOG_LEVEL_FLOW);        	
		
		String docFileName = outputDirName + File.separator + docName + ".xml";
		PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
		try
		{
			URL serviceURL = new URL(this.localShortURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);

	        Logger.sendMessage("DynamicRepository.addDocument before service.addDocument", Logger.LOG_LEVEL_FLOW);        	
			service.addDocument(docName, docFileName);
	        Logger.sendMessage("DynamicRepository.addDocument after service.addDocument", Logger.LOG_LEVEL_FLOW);        	

	        this.documents.add(docName);
		}
		catch(Exception e)
		{
			Logger.sendMessage("Failed to add dynamic AXML document " + docFileName + ".",
							   Logger.LOG_LEVEL_ERROR);        	
		}

        Logger.sendMessage("DynamicRepository.addDocument ended", Logger.LOG_LEVEL_FLOW);        			
	}

	public void updateDocument(String docName)
	{
        Logger.sendMessage("DynamicRepository.updateDocument started", Logger.LOG_LEVEL_FLOW);        	

        String docFileName = outputDirName + File.separator + docName + ".xml";
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

        Logger.sendMessage("DynamicRepository.updateDocument after cumentBuilderFactory.newInstance", Logger.LOG_LEVEL_FLOW);        	
		
		Document doc = null; 
		try
		{
			doc = dbf.newDocumentBuilder().parse(new File(docFileName));
			Element docElem = doc.getDocumentElement();

	        Logger.sendMessage("DynamicRepository.updateDocument after doc.getDocumentElement", Logger.LOG_LEVEL_FLOW);        	
			
			PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
			URL serviceURL = new URL(localShortURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);
	        
			Logger.sendMessage("DynamicRepository.updateDocument after locator.getPeerAccessPoint", Logger.LOG_LEVEL_FLOW);        	
			
			service.saveDocument(docName, true /* update */, docElem);
		}
		catch(Exception e)
		{
			Logger.sendMessage("ERROR: Failed to update dynamic AXML document " + docFileName + ".",
							   Logger.LOG_LEVEL_ERROR);        	
		}
		
        Logger.sendMessage("DynamicRepository.updateDocument ended", Logger.LOG_LEVEL_FLOW);        			
	}
	
	public void addService(String svcName)
	{
		String svcFileName = outputDirName + File.separator + svcName + ".xml";
		PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
		try
		{
			URL serviceURL = new URL(localShortURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);
			service.addService(svcName, svcFileName);
			this.services.add(svcName);
		}
		catch(Exception ex)
		{
			Logger.sendMessage("ERROR: Failed to add dynamic AXML service " + svcFileName + ".",
							   Logger.LOG_LEVEL_ERROR);        	
		}
	}
	
	public void flush()
	{
	}
	
	public void clear()
	{
		removeAllDocuments();
		removeAllServices();
	}
	
	private void removeAllDocuments()
	{
		String[] docNames = convertSetToStringArray(this.documents);
		this.documents.clear();
		
		PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
		try
		{
			URL serviceURL = new URL(localShortURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);
			service.removeDocuments(docNames);
		}
		catch(Exception ex)
		{
			Logger.sendMessage("ERROR: Failed to remove dynamic AXML documents. (System error: " + ex.getMessage() + ")",
							   Logger.LOG_LEVEL_ERROR);        	
		}		
	}

	private void removeAllServices()
	{
		String[] svcNames = convertSetToStringArray(this.services);
		this.services.clear();

		PeerAccessPointServiceLocator locator = new PeerAccessPointServiceLocator();
		try
		{
			URL serviceURL = new URL(localShortURL + "/services/PeerAccessPoint");
			localhost.axml.services.PeerAccessPoint.PeerAccessPoint service = locator.getPeerAccessPoint(serviceURL);
			service.removeServices(svcNames);
		}
		catch(Exception ex)
		{
			Logger.sendMessage("ERROR: Failed to remove dynamic AXML services. (System error: " + ex.getMessage() + ")",
							   Logger.LOG_LEVEL_ERROR);        	
		}		
	}
	
	private String[] convertSetToStringArray(Set set)
	{
		String[] strArray = new String[set.size()];
		Iterator setIter = set.iterator();
		int index = 0;
		while(setIter.hasNext())
		{
			strArray[index] = (String)setIter.next();
			index++;
		}
		
		return strArray;
	}
}
