package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

/**
 * @author pettelno
 *
 * A registry of AXML documents and services. Used only in a static mode
 * by the class StaticRepository to register new AXML documents and services
 * in docRepository.xml and svcRepository.xml.
 * 
 */
class RepositoryRegistry
{
	Element root = null;
	Document doc = null;
	String outputFileName = null;
	
	RepositoryRegistry(String outputFileName)
	{
        this.outputFileName = outputFileName;
		
		// First check if repository already exists.
        File repositoryFile = new File(outputFileName);
        if(repositoryFile.exists())
        {
        	SAXBuilder builder = new SAXBuilder();
        
	        try
			{
	        	this.doc = builder.build(repositoryFile);
	        	this.root = this.doc.getRootElement();
			}
	        catch(Exception e)
			{
	        	this.doc = null;
	        	this.root = null;
			}
        }
		
        // If repository doesn't exist, or if failed to use the existing
        // repository.
        if(this.doc == null || this.root == null)
        {
			root = new Element("mxbase");
	        root.setAttribute("origURI", "yaunos.inria.fr");
	        root.setAttribute("lastId", "0");       
	        doc = new Document(root);
        }
	}
	
	void addDocument(String docName)
	{
		Element forestElem = new Element("forest");
		forestElem.setAttribute("name", docName);
		forestElem.setAttribute("replicationIndex", "0");
		Element fileElement = new Element("file");
		fileElement.setAttribute("name", "");
		fileElement.setAttribute("href", docName + ".xml");
		forestElem.addContent(fileElement);
		root.addContent(forestElem);
	}
	
	void flush() throws IOException
	{
        FileOutputStream out = new FileOutputStream(outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(doc, out);
        out.flush();
        out.close();
	}	
}
