package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

/**
 * @author pettelno
 *
 * The builder of the activity file, "activity.xml", which is needed for
 * the termination detection algorithm, in a static mode. In this mode the
 * activity file is completely generated before the AXML peer starts 
 * running, as the list of relations is known in advance.
 * 
 */
class StaticActivityFile implements IActivityFile
{
	private Element root = null;
	private Document outDoc = null;
	private String outputFileName = null;
	
	StaticActivityFile(String outputDirName) throws IOException
	{
        this.root = new Element("activity").setAttribute("terminated", "n");
        this.outDoc = new Document(root);

        // Serialize it into a file.
        this.outputFileName = outputDirName + File.separator + "activity.xml";
        FileOutputStream out = new FileOutputStream(outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(this.outDoc, out);
        out.flush();
        out.close();
	}
	
	public void addRelation(String relationName) throws IOException
	{
		Element relationElem = new Element("relation").setAttribute("name", relationName);
		relationElem.addContent(new Element("activityLevel").setText("0"));
		
		this.root.addContent(relationElem);
		flush();
	}
	
	private void flush() throws IOException
	{
        FileOutputStream out = new FileOutputStream(this.outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(this.outDoc, out);
        out.flush();
        out.close();		
	}	
}
