package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import java.util.Iterator;
import java.util.List;
import il.ac.tau.cs.dqsq.common.*;

/**
 * @author pettelno
 *
 * This class represents a neighbors file, which is the file "neighbors.xml"
 * that is generated for every AXML peer. This file is used for the 
 * termination detection algorithm. It contains the list of relations (nodes)
 * and for each relation, the list of its neighbors, meaning the relations
 * it invokes directly.
 * The StaticNeighborsFile is used only by the StaticPeer class.
 * 
 */
class StaticNeighborsFile implements INeighborsFile
{
	private Element root = null;
	private Document outDoc = null;
	String outputFileName = null;

	StaticNeighborsFile(String outputDirName, String peerName) throws IOException
	{
        this.root = new Element("peer").setAttribute("name", peerName);
		
        this.outDoc = new Document(root);

        // Serialize it into a file.
        this.outputFileName = outputDirName + File.separator + "neighbors.xml";
        FileOutputStream out = new FileOutputStream(outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(this.outDoc, out);
        out.flush();
        out.close();
	}
	
	public void addRelation(String relationName, List neighbors) throws IOException
	{
		boolean isNewRelation = false;
		Element relationElem = getRelationElem(relationName);
		if(relationElem == null)
		{
			relationElem = new Element("relation").setAttribute("name", relationName);
			isNewRelation = true;
		}
		
        if(neighbors != null)
        {
			Iterator neighborsIter = neighbors.iterator();
	        while(neighborsIter.hasNext())
	        {
	        	RelationOccurrence curNeighborRel = (RelationOccurrence)neighborsIter.next();
	        	String neighborName = curNeighborRel.getAdornedName();
	        	Element neighborElem = relationElem.getChild(neighborName);
	        	if(neighborElem == null)
	        	{
	        		neighborElem = new Element("neighbor");
	        		neighborElem.setAttribute("name", neighborName);
	        		neighborElem.setAttribute("peer", curNeighborRel.getPeerName());
	        		neighborElem.setAttribute("url", curNeighborRel.getServiceShortURL());
	        		
	        		relationElem.addContent(neighborElem);
	        	}
	        }
        }        
		
		if(isNewRelation)
			this.root.addContent(relationElem);
		
		flush();
	}
	
	void flush() throws IOException
	{
        FileOutputStream out = new FileOutputStream(this.outputFileName);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(this.outDoc, out);
        out.flush();
        out.close();		
	}		
	
	private Element getRelationElem(String relationName)
	{
		List relationElems = this.root.getChildren();
		Element curElem = null;
		Iterator iter = relationElems.iterator();
		while(iter.hasNext())
		{
			curElem = (Element)iter.next();
			if(curElem.getAttributeValue("name").equals(relationName))
				return curElem;
		}
		
		return null;
	}
}
