
package il.ac.tau.cs.dqsq.qsq2axml;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

/**
 * @author pettelno
 *
 * This class represents a Peer when running in a static mode, meaning when
 * QSQ2AXML is running as a standalone application (for testing purpose).
 * It reads the Datalog input and generates the corresponding AXML documents
 * and services. Note that in a static mode all the rules and relations
 * are known to the peer in advance, thus translated all at once when the
 * run methos is called.
 * 
 */
public class StaticPeer extends Peer
{
	private String inputFileName = null;
	
	StaticPeer(String inputFileName, String outputDirName) throws IOException, JDOMException
	{
		super(outputDirName);
		this.inputFileName = inputFileName;
    	this.axmlRepository = new StaticRepository(outputDirName);
	}

    void run() throws Exception
    {
    	this.activityFile = new StaticActivityFile(this.outputDirName);
    	this.neighborsFile = new StaticNeighborsFile(this.outputDirName, this.peerName);
    	readInput(this.inputFileName);
    	super.run();
    }

    private void readInput(String inputFileName) throws Exception
    {
        // Build the document with SAX and Xerces, no validation.
        SAXBuilder builder = new SAXBuilder();
        Document doc = builder.build(new File(inputFileName));
        Element root = doc.getRootElement();

    	this.peerName = root.getAttributeValue("peer");
        
        // Read general settings.
//        generalSettingsElem = root.getChild("generalSettings");
        
        // Read relations.
        Element relationsElem = root.getChild("relations");
        if(relationsElem != null)
            readRelations(relationsElem);

        // Read rules.
        Element rulesElem = root.getChild("rules");
        if(rulesElem != null)
            readRules(rulesElem);
    }

    private void readRelations(Element relationsElem) throws Exception
    {
        List relationElems = relationsElem.getChildren();
        Iterator relationsIter = relationElems.iterator();
        while(relationsIter.hasNext())
        {
            Element relationElem = (Element)relationsIter.next();
            RelationTranslator curRelation = new RelationTranslator(
				this.axmlRepository, this.peerName);
            curRelation.read(relationElem);
            relations.put(relationElem.getAttributeValue("name"), curRelation);
        }
    }

    private void readRules(Element rulesElem) throws Exception
    {
        List ruleElems = rulesElem.getChildren();
        Iterator rulesIter = ruleElems.iterator();
        while(rulesIter.hasNext())
        {
            Element ruleElem = (Element)rulesIter.next();
            RuleTranslator curRule = new RuleTranslator(this.axmlRepository, this.peerName);
            curRule.read(ruleElem);
            rules.add(curRule);
        }
    }    
}
