package il.ac.tau.cs.dqsq.termdetection;

import il.ac.tau.cs.dqsq.util.dQSQUtils;
import il.ac.tau.cs.dqsq.termdetection.ActivityService;

/**
 * @author pettelno
 *
 * Hosts the recvAck web service as a part of the acknowledgements 
 * management, which is required by the termination detection algorithm.
 * 
 */
public class AckService
{
	/**
	 * This method is exposed as a web service. Node A calls recvAck on
	 * node B when A acknowledges a message that it previously received
	 * from B. 
	 * 
	 * @param relationName The name of the relation to receive the ack. 
	 * @param from The name of the relation that sends the ack.
	 * @param receivingPeer The name of the peer that hosts the
	 *  	  relation that receive the ack.
	 * @param ackReason The reason for sending the ack (for debug purposes).
	 */
	public void recvAck(String relationName, String from, String receivingPeer, String ackReason)
	{
		dQSQUtils.debug("Relation " + relationName + " (receiving peer: " + receivingPeer + ") received ack from " + from + " - " + ackReason, "debug_AckService.txt");

		// Decrement activity value.
		new ActivityService().decrementActivity(relationName, "after receiving ack from " + from + " - " + ackReason, receivingPeer);
	}
}
