package il.ac.tau.cs.dqsq.termdetection;

import java.net.URL;
import localhost.axml.services.IdlenessManager.IdlenessManagerServiceLocator;
import il.ac.tau.cs.dqsq.util.dQSQUtils;

/**
 * @author pettelno
 *
 * A node in the termination detection (TD) algorithm, which is remote
 * (representing a node that is located on a remote peer).
 * In our implementation a node in the TD algorithm is a dQSQ relation. 
 * 
 */
class RemoteRelationIdlenessManager extends RelationIdlenessManager
{
	String relationURL;
	String relationPeer;
	
	public RemoteRelationIdlenessManager(String relationName, String relationURL, String relationPeer)
	{
		this.relationName = relationName;
		this.relationURL = relationURL;
		this.relationPeer = relationPeer;
	}
	
	public int isIdle(int cycleID, String peer /* unused */)
	{
		dQSQUtils.debug("DEBUG - Entered RemoteRelationIdlenessManager.isIdle. Relation: " + relationName + ". URL: " + relationURL, "debug_Idleness.txt");					
		
		// Call the isIdle web service on the remote peer.
		IdlenessManagerServiceLocator locator = new IdlenessManagerServiceLocator();
		try
		{
			URL serviceURL = new URL(relationURL + "/services/IdlenessManager");
			localhost.axml.services.IdlenessManager.IdlenessManager service = locator.getIdlenessManager(serviceURL);
			return service.isIdle(relationName, cycleID, relationPeer);
		}
		catch(Exception e)
		{
			dQSQUtils.debug("ERROR - RemoteRelationIdlenessManager.isIdle: Relation " + relationName + " failed to invoke remote service isIdle. URL: " + relationURL, "debug_Idleness.txt");					
			return Constants.STATUS_ACTIVE;
		}
	}

	public void terminateServices()
	{
		// Call the terminateServices web service on the remote peer.
		IdlenessManagerServiceLocator locator = new IdlenessManagerServiceLocator();
		try
		{
			URL serviceURL = new URL(relationURL + "/services/IdlenessManager");
			localhost.axml.services.IdlenessManager.IdlenessManager service = locator.getIdlenessManager(serviceURL);
			service.terminateServices(relationPeer);
		}
		catch(Exception e)
		{
			dQSQUtils.debug("ERROR - RemoteRelationIdlenessManager.terminateServices: Relation " + relationName + " failed to invoke remote service terminateServices.", "debug_Idleness.txt");					
		}		
	}
}
