package il.ac.tau.cs.dqsqenv;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.DocType;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import il.ac.tau.cs.dqsq.util.FileUtils;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * @author pettelno
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
class Peer
{
	private String axmlHomeDir = null;
	private String axmlRepositoryDir = null;
	private String datalogInputFile = null;
	private String axmlWebappName = null;
	private String peerName = null;
	
	Peer(Element peerElem)
	{
		this.axmlHomeDir = peerElem.getChildText("axmlHomeDir");
		this.axmlRepositoryDir = this.axmlHomeDir + File.separatorChar + "repository";
		this.datalogInputFile = peerElem.getChildText("datalogInputFile");
	}
	
	void initRepository() throws IOException
	{
		// Clean the repository.
		FileUtils.deleteFilesFromDir(new File(this.axmlRepositoryDir));
		
		// Copy empty docRepository.xml and svcRepository.xml to the repository.
		FileUtils.copyDir(new File("setup" + File.separatorChar + "repository"),
						  new File(this.axmlRepositoryDir));

		// Copy datalog input file. The file is renamed datalog.xml.
		if(isActive())
			FileUtils.copyFile(new File(this.datalogInputFile),
							   new File(this.axmlRepositoryDir + File.separatorChar + "datalog.xml"));
	}	
	
	void initLogDir()
	{
		// Clean the dQSQ log dir. 
		FileUtils.deleteFilesFromDir(new File(this.axmlHomeDir +
				File.separatorChar + "dQSQ" +
				File.separatorChar + "log"));
	}
	
	public String getAxmlHomeDir()
	{
		return this.axmlHomeDir;
	}
	
	public void install() throws Exception
	{
		// Copy the setup dir to the AXML installation.
		FileUtils.copyDirRecursive(new File("setup"), new File(this.axmlHomeDir));
		
		fixURLInWebXml();
	}
	
	private void fixURLInWebXml() throws Exception
	{
		// Set the correct URL in web.xml. 
		String webXmlFilePath = this.axmlHomeDir + File.separatorChar + 
								"WEB-INF" + File.separatorChar + "web.xml";
		
		SAXBuilder builder = new SAXBuilder();     
		Document doc = builder.build(new File(webXmlFilePath));
		
		Element root = doc.getRootElement();
		List servletElems = root.getChildren("servlet");
		int numOfServlets = servletElems.size();
		for(int servletIndex = 0; servletIndex < numOfServlets; servletIndex++)
		{
			Element servletElem = (Element)servletElems.get(servletIndex);
			if(servletElem.getChildText("servlet-name").equals("AXMLServlet"))
			{
				List initParamElems = servletElem.getChildren("init-param");
				int numOfInitParams = initParamElems.size();
				for(int initParamIndex = 0; initParamIndex < numOfInitParams; initParamIndex++)
				{
					Element initParamElem = (Element)initParamElems.get(initParamIndex);
					if(initParamElem.getChildText("param-name").equals("axml.peer.url"))
						initParamElem.getChild("param-value").setText(
								"http://" + getLocalHostName() + ":8080/" + getAxmlWebappName());
				}				
			}
		}
		
		// Set the doc type.
		DocType docType = new DocType("web-app", "-//Sun Microsystems, Inc.//DTD Web Application 2.2//EN", "http://java.sun.com/j2ee/dtds/web-app_2.2.dtd");
		doc.setDocType(docType);
		
        // Save the updated file.
        FileOutputStream out = new FileOutputStream(webXmlFilePath);
        XMLOutputter serializer = new XMLOutputter();
        serializer.output(doc, out);
        out.flush();
        out.close();		
	}
	
	private String getLocalHostName() throws UnknownHostException
	{
		InetAddress addr = InetAddress.getLocalHost();  
		return addr.getHostName();
	}
	
	private String getAxmlWebappName()
	{
		if(this.axmlWebappName == null)
		{
			String[] splittedAxmlHomeDir = this.axmlHomeDir.split("webapps");		
			
			// Return the webapp name without the slash in the beginning.
			this.axmlWebappName = splittedAxmlHomeDir[1].substring(1);
		}
		
		return this.axmlWebappName;
	}
	
	public Element createHtmlLinkToHomePage() throws Exception	 
	{
		if(!isActive())
			return null;
				
		Element linkElem = new Element("a");
		linkElem.setAttribute("href", "http://localhost:8080/" +
							  getAxmlWebappName() + "/jsp/dQSQ/dQSQ.jsp");
		linkElem.setAttribute("target", "_blank");
		
		Element fontElem = new Element("font").setAttribute("size", "4");
		fontElem.setText("Peer " + getPeerName());
		linkElem.addContent(fontElem);
		
		return linkElem;		
	}
	
	private String getPeerName() throws Exception
	{
		if(this.peerName == null)
		{
			// Resolve the peer name from the Datalog input file.
			SAXBuilder builder = new SAXBuilder();     
			Document doc = builder.build(new File(this.datalogInputFile));
			
			Element root = doc.getRootElement();
			this.peerName = root.getAttributeValue("peer");
		}
		
		return this.peerName;
	}
	
	public boolean isActive()
	{
		// A peer is active if there is a Datalog input file associated with it.
		return (this.datalogInputFile != null &&
				!this.datalogInputFile.equals(""));
	}
}
